// Copyright 2016, University of Colorado Boulder

(function() {
  'use strict';

  var radioGroupName = 'predict';

  /**
   * Enables the submit button if true
   * @returns {boolean} - true if radio button is selected and textarea is not empty
   */
  function validatePredict() {
    // Check that there is a radio button selected
    var flag = false;
    $( 'input[name="' + radioGroupName + '"]' ).each( function() {
      if ( !flag && this.checked ) {
        flag = true;
      }
    } );
    // this checks if the text area has been filled out
    flag = flag && !!$( '#predict-input' ).val();

    // Checks that the sim is loaded;
    flag = flag && window.simLaunched;

    // Enable/Disable the submit button if needed
    var submitButton = document.getElementById( 'predict-submit' );
    if ( flag && submitButton.classList.contains( 'disabled' ) ) {
      submitButton.classList.remove( 'disabled' );
    }
    else if ( !flag && !submitButton.classList.contains( 'disabled' ) ) {
      submitButton.classList.add( 'disabled' );
    }

    return flag;
  }

  /**
   * For the predict tab: checks that all form elements are complete,
   * disables them, removes the submit button, and enables the explore tab.
   * @returns {boolean} - false always.  This allows button to be pickable in the form but does not submit a post request
   */
  function onPredictSubmit() {

    if ( validatePredict() ) {
      // Disable radio buttons and textarea
      $( 'input[name="' + radioGroupName + '"]' ).attr( 'disabled', 'disabled' );
      $( '#predict-input' ).attr( 'disabled', 'disabled' );

      // Highlight next tab
      $( '#predict-next' ).show();
      $( '#predict-submit' ).hide();
      $( '#explore' ).attr( 'class', 'tab pickable' );

      window.recordButtonPressed( this );

    }
    else {
      window.toast( 'Prediction and explanation are required to proceed.' );
    }
    return false;
  }

  /**
   * Life cycle functions for this tab.
   * @type {{initialize: tabs.experiment.initialize, onLoad: tabs.experiment.onLoad, onLeave: tabs.experiment.onLeave, afterStateLoad: tabs.experiment.afterStateLoad}}
   */
  window.tabs.predict = {
    /**
     * Initial set up of the tab. Sets the click listeners and prepares the sim accordingly
     * Called from classroom-activity.switchTab.
     */
    initialize: function() {
      $( '#predict-submit' ).click( onPredictSubmit );

      window.createHypothesisRadioGroup( document.getElementById( 'predict-radio-group' ), window.simConfiguration.hypothesisChoices, radioGroupName );

      // Listener for radio buttons on the predict tab
      $( 'input[name=' + radioGroupName + ']' ).click( function() {
        window.classroom.predict.prediction = this.value;
        window.recordRadioButtonFired( this );
        validatePredict();
        return true;
      } );

      // Listener for textarea changes on the predict tab
      $( '#predict-input' ).on( 'keyup change paste', function() {
        // Record text area only if it changed
        window.classroom.predict.explanation = window.recordTextAreaChanged( this, window.classroom.predict.explanation );
        validatePredict();
        return true;
      } );

      $( '#predict-text-1' ).text( window.simConfiguration.predictText1 );
      $( '#predict-text-2' ).text( window.simConfiguration.predictText2 );

      // Wait for the sim to be launched
      var predictInterval = setInterval( function() {
        if ( window.simLaunched ) {
          clearInterval( predictInterval );
          validatePredict();
        }
      }, 100 );

    },

    /**
     * Sets the properties of the classroom object and customizations of the sim
     * that must be done each time traveling to this tab.
     * Called from classroom-activity.switchTab whenever going to this tab
     */
    onLoad: function() {},
    /**
     * Called from classroom-activity.switchTab whenever leaving this tab and before loading/initializing the next
     */
    onLeave: function() {},
    /**
     * Called from classroom-activity.switchTab after onLoad and this tab's customizations have been invoked and loaded
     */
    afterStateLoad: function() {}
  };
})();

