// Copyright 2016, University of Colorado Boulder

(function() {
  'use strict';

  /**
   * Life cycle functions for this tab.
   * @type {{initialize: tabs.experiment.initialize, onLoad: tabs.experiment.onLoad, onLeave: tabs.experiment.onLeave, afterStateLoad: tabs.experiment.afterStateLoad}}
   */
  window.tabs.reflect = {
    /**
     * Initial set up of the tab. Sets the click listeners and prepares the sim accordingly
     * Called from classroom-activity.switchTab.
     */
    initialize: function() {
      window.classroom.reflect.dataTable = new window.DataTable( window.simConfiguration.recordDataPoint,
        window.simIFrameClient, 'classroomActivity', window.simConfiguration.experimentTableHeaders, 'reflect-table' );

      $( '#reflect-input' ).on( 'keyup change paste', function() {

        // Record text area only if it changed
        window.classroom.reflect.rules = window.recordTextAreaChanged( this, window.classroom.reflect.rules );

        var $rb = $( '#reflect-button-save' );
        if ( this.value.length > 0 ) {
          if ( $rb.hasClass( 'disabled' ) ) {
            $rb.removeClass( 'disabled' );
          }
        }
        else {
          if ( !$rb.hasClass( 'disabled' ) ) {
            $rb.addClass( 'disabled' );
          }
        }
      } );

      $( '#reflect-button-save' ).click( function() {
        if ( !$( this ).hasClass( 'disabled' ) ) {
          window.classroom.apply.rules = window.classroom.reflect.rules;
          $( this ).hide();
          $( '#reflect-button-edit' ).show();

          $( '#reflect-input' ).attr( 'disabled', 'disabled' );

          if (window.classroom.reflect.firstTimeSaving) {
            $( '#apply' ).toggleClass( 'disabled pickable' );
            $( '#reflect-next' ).show();
            window.classroom.reflect.active = false;
            window.classroom.reflect.firstTimeSaving = false;
          }

          window.recordButtonPressed( this );
        }
        else {
          window.toast( 'Please record some rules for which team wins' );
        }
      } );

      var $reflectEditButton = $( '#reflect-button-edit' );

      // Hide it on load
      $reflectEditButton.hide();

      $reflectEditButton.click( function() {
        $( this ).hide();
        $( '#reflect-button-save' ).show();
        $( '#reflect-input' ).prop( 'disabled', false );
      } );

      $( '#reflect-text-1' ).text( window.simConfiguration.reflectText1 );
      $( '#reflect-text-2' ).text( window.simConfiguration.reflectText2 );
      $( '#reflect-text-3' ).text( window.simConfiguration.reflectText3 );
      this.onLoad();
    },

    /**
     * Sets the properties of the classroom object and customizations of the sim
     * that must be done each time traveling to this tab.
     * Called from classroom-activity.switchTab whenever going to this tab
     */
    onLoad: function() {
      window.simIFrameClient.invokeSequence( [
        { phetioID: window.simConfiguration.modelTandem, method: 'reset' },
        {
          phetioID: window.simConfiguration.activePropertyTandem,
          method: 'setValue',
          args: [ window.classroom.reflect.active ]
        },
        { phetioID: 'phetio', method: 'setInteractive', args: [ window.classroom.reflect.active ] }
      ] );

      window.drawDataTable( window.classroom.reflect.dataTable, 'reflect' );

      if(!window.classroom.reflect.active){
        // Hide ability to edit if you've moved on
        $( '#reflect-button-edit' ).hide();
        $( '#phet-data-table-record').addClass('disabled');

      }
    },

    /**
     * Called from classroom-activity.switchTab whenever leaving this tab and before loading/initializing the next
     */
    onLeave: function() {
      document.getElementById( 'data-table' ).innerHTML = '';
      var recordDiv = document.getElementById( 'reflect-workspace' );
      recordDiv.removeChild( document.getElementById( 'phet-data-table-record' ) );

      //
      if(!window.classroom.reflect.active){
        // Save input
        $( '#reflect-button-save' ).click();
      }
    },

    /**
     * Called from classroom-activity.switchTab after onLoad and this tab's customizations have been invoked and loaded
     */
    afterStateLoad: function() {}
  };
})();
