// Copyright 2016, University of Colorado Boulder

// If you include Toast in your project you must include Toast.css as well.

// Create an invisible popup element and append to the DOM.
var popup = document.createElement( 'div' );
popup.setAttribute( 'class', 'popup-message' );
popup.setAttribute( 'id', 'popup' );
popup.style.display = 'none';
var dismissToastButton = document.createElement( 'button' );
dismissToastButton.innerHTML = 'OK';
var toastText = document.createElement( 'p' );
popup.appendChild( toastText );
popup.appendChild( dismissToastButton );
document.body.appendChild( popup );

var shouldDismiss = false;
window.dismissTimeLeft = 0;

/**
 * Shows a message in a popup box. If timeout is 0 or omitted, the popup box prompts the user to click ok to dismiss.
 * To alter the location and appearance of the popup, edit the css rules in lab-book.css for div.popup-message
 *
 * @param message:String
 * @param Option timeout:Number - ms to dismiss message.  Defaults to 0 (prompt to dismiss).
 * @param Option buttonText:String - text to show in dismiss button if timeout is 0. Defaults to OK.
 */
window.toast = function( message, timeout, buttonText ) {
  'use strict';
  var $popup = $( '#popup' );
  var $text = $( '#popup p' );
  var $button = $( '#popup button' );

  var promptToDismiss = !timeout;
  if ( promptToDismiss ) {
    if ( !buttonText ) {
      buttonText = 'OK';
    }
    $button.show();

  }
  else {
    $button.hide();
  }

  $text.text( message );
  $button.text( buttonText );
  if ( !$popup.is( ':visible' ) ) {
    $popup.toggle( 'puff' );
  }

  if ( !promptToDismiss ) {

    if ( window.dismissTimeLeft === 0 || !shouldDismiss ) {
      shouldDismiss = true;
      window.dismissTimeLeft = timeout;
      var toastInterval = setInterval( function() {
        window.dismissTimeLeft -= 100;
        if ( shouldDismiss && window.dismissTimeLeft <= 0 ) {
          clearInterval( toastInterval );
          shouldDismiss = false;
          $popup.fadeOut();
        }

        if ( !shouldDismiss ) {
          clearInterval( toastInterval );
        }
      }, 100 );
    }
    else {
      window.dismissTimeLeft = timeout;
    }
  }
  else {
    shouldDismiss = false;
  }
};

$( '#popup button' ).click( function() {
  'use strict';

  var $popup = $( '#popup' );
  if ( $popup.is( ':visible' ) ) {
    $popup.fadeOut();
  }
} );