// Copyright 2016, University of Colorado Boulder

// Loaded by lab-book.js
(function() {
  'use strict';

  // specify the solution for the lab with query parameter
  var solution = QueryStringMachine.get( 'solution', {
    type: 'string',
    defaultValue: 'copperSulfate',
    validValues: [ 'drinkMix', 'cobaltIINitrate', 'cobaltChloride', 'potassiumDichromate', 'potassiumChromate', 'nickelIIChloride', 'copperSulfate', 'potassiumPermanganate', 'sodiumChloride' ]
  } );

  function wavelengthToColor( wavelength ) {
    // Reused from /scenery-phet/js/VisibleColor.js
    if ( wavelength < 380 || wavelength > 780 ) {
      return '#ffffff';
    }

    var r;
    var g;
    var b;

    // Determine the RGB component values.
    if ( wavelength >= 380 && wavelength <= 440 ) {
      r = -1 * ( wavelength - 440 ) / ( 440 - 380 );
      g = 0;
      b = 1;
    }
    else if ( wavelength > 440 && wavelength <= 490 ) {
      r = 0;
      g = ( wavelength - 440 ) / ( 490 - 440 );
      b = 1;
    }
    else if ( wavelength > 490 && wavelength <= 510 ) {
      r = 0;
      g = 1;
      b = -1 * ( wavelength - 510 ) / ( 510 - 490 );
    }
    else if ( wavelength > 510 && wavelength <= 580 ) {
      r = ( wavelength - 510 ) / ( 580 - 510 );
      g = 1;
      b = 0;
    }
    else if ( wavelength > 580 && wavelength <= 645 ) {
      r = 1;
      g = -1 * ( wavelength - 645 ) / ( 645 - 580 );
      b = 0;
    }
    else if ( wavelength > 645 && wavelength <= 780 ) {
      r = 1;
      g = 0;
      b = 0;
    }

    // Let the intensity fall off near the vision limits.
    var intensity;
    if ( wavelength > 645 ) {
      intensity = 0.3 + 0.7 * ( 780 - wavelength ) / ( 780 - 645 );
    }
    else if ( wavelength < 420 ) {
      intensity = 0.3 + 0.7 * ( wavelength - 380 ) / ( 420 - 380 );
    }
    else {
      intensity = 1;
    }
    var red = Math.round( 255 * ( intensity * r ) );
    var green = Math.round( 255 * ( intensity * g ) );
    var blue = Math.round( 255 * ( intensity * b ) );

    return 'rgb(' + red + ',' + green + ',' + blue + ')';
  }

  window.simConfiguration = {

    pageTitle: 'Light Absorbance Lab',
    simTitle: 'Light Absorbance Lab',
    simName: 'beersLawLab',

    customization: [ {
      phetioID: 'beersLawLab.beersLawScreen.view.solutionControls.comboBox',
      method: 'setPickable',
      args: [ false ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.model.solutionProperty',
      method: 'setValue',
      args: [ 'beersLawLab.beersLawScreen.solutions.' + solution ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.view.wavelengthControls.presetWavelengthRadioButton',
      method: 'setVisible',
      args: [ false ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.view.wavelengthControls.variableWavelengthRadioButton',
      method: 'setVisible',
      args: [ false ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.view.wavelengthControls.variableWavelengthProperty',
      method: 'setValue',
      args: [ true]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.model.ruler.locationProperty',
      method: 'setValue',
      args: [ { x: 3.3, y: 3.58 } ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.view.wavelengthControls.valueDisplay',
      method: 'setVisible',
      args: [ false ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.view.wavelengthControls.valueBackground',
      method: 'setVisible',
      args: [ false ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.model.detector.modeProperty',
      method: 'setValue',
      args: [ 'absorbance' ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.view.detectorNode.bodyNode.transmittanceRadioButton',
      method: 'setVisible',
      args: [ false ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.view.detectorNode.bodyNode.absorbanceRadioButton',
      method: 'setCircleButtonVisible',
      args: [ false ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.view.solutionControls.comboBox.arrow',
      method: 'setVisible',
      args: [ false ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.view.solutionControls.comboBox.separator',
      method: 'setVisible',
      args: [ false ]
    }, {
      phetioID: 'beersLawLab.beersLawScreen.view.resetAllButton',
      method: 'setVisible',
      args: [ false ]
    } ],

    // To add a listener to reset the customizations on reset.
    resetAllButtonFiringPropertyTandem: 'beersLawLab.beersLawScreen.view.resetAllButton.isFiringProperty',

    // Table column header text
    tableHeaders: [
      { text: 'Wavelength' },
      { text: 'Width (cm)' },
      // TODO: Make this unit change dynamic based on the solution range
      // { text: 'Concentration (&mu;M)' },
      { text: 'Concentration (mM)' },
      { text: 'Absorbance' }
    ],

    // TODO: are these used?  Probably not.
    queryString: 'cuvetteSnapInterval=0&screens=2',

    features: [
      // TODO: make this unit change dynamic based on the solution range
      // { value: 'concentration', content: 'Concentration (\u03bcM)' },
      { value: 'trialNumber', content: 'Trial Number' },
      { value: 'concentration', content: 'Concentration (mM)' },
      { value: 'absorbance', content: 'Absorbance' },
      { value: 'cuvetteWidth', content: 'Width (cm)' }
    ],

    addCells: function( row, dataPoint ) {
      var cell1 = row.insertCell( 1 );
      if ( dataPoint.wavelength !== null ) {
        var wavelengthDot = document.createElement( 'i' );
        wavelengthDot.style.color = wavelengthToColor( dataPoint.wavelength );
        wavelengthDot.setAttribute( 'aria-hidden', 'true' );
        wavelengthDot.setAttribute( 'class', 'fa fa-circle' );
        cell1.appendChild( wavelengthDot );
      }
      else {
        cell1.innerHTML = '-';
      }

      row.insertCell( 2 ).innerHTML = Number( dataPoint.cuvetteWidth ).toFixed( 2 );
      row.insertCell( 3 ).innerHTML = Number( dataPoint.concentration ).toFixed( 2 );

      // Absorbance should be blank instead of 0.0 when not measuring.
      if (dataPoint.wavelength === null || dataPoint.absorbance === null) {
        row.insertCell( 4 ).innerHTML = '-';
      }
      else{
        row.insertCell( 4 ).innerHTML = Number( dataPoint.absorbance ).toFixed( 2 );
      }
    },

    recordDataPoint: function( simIFrameClient, callback ) {
      simIFrameClient.invokeSequence( [
          { phetioID: 'beersLawLab.beersLawScreen.model.light.wavelengthProperty', method: 'getValue' },
          { phetioID: 'beersLawLab.beersLawScreen.model.cuvette.widthProperty', method: 'getValue' },
          { phetioID: 'beersLawLab.beersLawScreen.model.solutionProperty', method: 'getValue' },
          { phetioID: 'beersLawLab.beersLawScreen.model.detector.valueProperty', method: 'getValue' }, //absorbance
          { phetioID: 'beersLawLab.beersLawScreen.model.light.onProperty', method: 'getValue' },
          { phetioID: 'beersLawLab.sim', method: 'getScreenshotDataURL' },
          { phetioID: 'phetio', method: 'getState' }
        ],
        function( wavelength, width, solution, absorbance, lightOn, screenshot, state ) {
          var image = document.createElement( 'img' );
          image.className = 'thumbnail';
          image.src = screenshot;

          var concentration = solution.concentrationProperty._value * 1000;

          var dataPoint = {
            wavelength: (lightOn) ? wavelength : null,
            cuvetteWidth: width,
            concentration: concentration,
            absorbance: absorbance,
            image: image,
            state: state,
            visible: false
          };

          callback( dataPoint );

        } );
    }
  };
})();