// Copyright 2016, University of Colorado Boulder

(function() {
  'use strict';

  var queryParameters = QueryStringMachine.getAll( {
    wrapper: { type: 'string' },
    validationRule: { type: 'string' }
  } );

  // Load the validation script, see http://stackoverflow.com/questions/14521108/dynamically-load-js-inside-js
  var script = document.createElement( 'script' );
  script.src = queryParameters.validationRule + '.js'; // creates window.validationRule

  document.head.appendChild( script );

  function submit() {
    var learnerID = textBox.value;

    var validation = window.validationRule( learnerID );

    if ( validation.error !== null ) {
      document.getElementById( 'errorMessage' ).innerHTML = validation.error;
    }
    else {

      // Pass the learner_id through the session_storage so it doesn't appear in the wrapper URL.  This will require
      // users to go through the login page before starting the wrapper.
      sessionStorage.setItem( 'phet.metacog.learner_id', learnerID );

      // Pass all query parameters through to sim
      var wrapper = queryParameters.wrapper;


      // This is to support requirejs mode only, use chipper's wrapper list
      if ( !WrapperUtils.isBuilt ) {

        // Request the master list of wrappers, and check to see if the requested wrapper is dedicated or in phet-io-wrappers/
        window.readFile( '../../chipper/data/wrappers', function( data ) {
          var wrappersList = data.split( '\n' );

          var path = '';
          wrappersList.forEach( function( wrapperPath ) {

            // Dedicated wrappers look like 'phet-io-wrapper-classroom-activity', with no slashes in the path.
            var isDedicatedWrapper = wrapperPath.split( '/' ).length === 1;

            // path to the dedicated wrapper
            if ( isDedicatedWrapper && wrapperPath.indexOf( wrapper ) >= 0 ) {
              path = '../../phet-io-wrapper-' + wrapper + '/' + wrapper + '.html';
            }
          } );

          // If the path is for a wrapper from the phet-io-wrappers suite. i.e. "phet-io-wrappers/active"
          path = path === '' ? '../' + wrapper + '/' + wrapper + '.html' : path;

          window.location = path + window.location.search + '&loginURL=' + encodeURIComponent( window.location.href );

        } );
      }
      else {
        window.location = '../' + wrapper + '/' + wrapper + '.html' + window.location.search +
                          '&loginURL=' + encodeURIComponent( window.location.href );
      }
    }
  }

  var textBox = document.getElementById( 'idInputBox' );

  textBox.addEventListener( 'input', function( evt ) {
    // Disable certain characters from the input area by replacing them with the empty string
    textBox.value = textBox.value.replace( /[ @]/g, '' );
  } );

  textBox.addEventListener( 'keydown', function( evt ) {
    // enter is pressed
    if ( evt.keyCode === 13 ) {
      submit( evt );
    }
  } );

  document.getElementById( 'launchButton' ).addEventListener( 'click', submit );
})();