// Copyright 2016, University of Colorado Boulder

/**
 * Here are the IDs for Ian/FSU function Builder group:
 U16001a
 U16002b
 ....
 U16100(appropriate alphabet letter if you just rotate through these a-z then back to a)

 F16001a again up to F16100(letter)

 C16001a again up to C16100(letter)

 R16001a again up to R16100(letter)

 These are basically 4 teachers/sites, 2016 school year, students 1-100, with the letter making it harder for students to mistype an ID. So there will be 400 acceptable IDs
 *
 * @param string
 * @returns {{error: null}|string} Object containing value error, which contains a message if the validation was not successful and is null otherwise
 */
window.validationRule = function( string ) {
  'use strict';

  /**
   *  Add a testing id for all validation rules: phettest
   */
  if ( string === 'phettest' ) {
    return { error: null };
  }

  var startLetter = [ 'U', 'F', 'C', 'R' ];
  if ( startLetter.indexOf( string[ 0 ] ) >= 0 ) {
    if ( string[ 1 ] === '1' && string[ 2 ] === '6' ) {
      var number = Number( string[ 3 ] + string[ 4 ] + string[ 5 ] );
      if ( Number.isInteger( number ) ) {
        if ( number > 0 && number <= 100 ) {

          // 97 is the ASCII code for lower case 'a'.
          var chr = String.fromCharCode( 97 + (number - 1 ) % 26 ); // where n is 0, 1, 2 ...
          if ( chr === string[ 6 ] ) {
            if ( string[ 7 ] === undefined ) {
              return { error: null };
            }
            else {
              return { error: 'Error: too many characters' };
            }
          }
          else {
            return { error: 'Error: wrong crosscheck' };
          }
        }
        else {
          return { error: 'Error: number out of range' };
        }
      }
      else {
        return { error: 'Error: number should be an integer' };
      }
    }
    else {
      return { error: 'Error: wrong year' };
    }
  }
  else {
    return { error: 'Error: wrong prefix for ' + string };
  }
};

(function() {
  'use strict';
  var shouldPass = function( string ) {
    if ( window.validationRule( string ) !== null ) {
      console.log( 'ERROR: ' + string );
    }
  };
  var shouldFail = function( string ) {
    if ( window.validationRule( string ) === null ) {
      console.log( 'ERROR: ' + string );
    }
  };
  // Automated tests
  shouldPass( 'U16001a' );
  shouldPass( 'U16002b' );
  shouldPass( 'F16001a' );
  shouldPass( 'C16001a' );
  shouldPass( 'R16001a' );
  shouldPass( 'F16100v' );
  shouldPass( 'C16100v' );
  shouldPass( 'R16100v' ); // testing one
  shouldPass( 'F16026z' );
  shouldPass( 'U16027a' );

  shouldFail( 'U16028a' );
  shouldFail( 'U16001b' );
  shouldFail( 'hello' );
  shouldFail( 'W16123b' );
  shouldFail( 'F16003a' );
  shouldFail( 'U16001a1' );
  shouldFail( '' );
})();
